;; Homeros Menu: The menu to start new work for Homeros distribution;
;; Michael Pozhidaev <msp@altlinux.org>
;; Date: 2011-04-27;

(defcustom homeros-menu-header "Choose the item from the list below:" "Homeros menu title shown at the buffer top")
(defcustom homeros-menu-message "Choose the item!" "The message shown at the screen bottom at Homeros menu appearance")
(defcustom homeros-menu-top-text 
(lambda () (insert "Current time is " (shell-command-to-string "LANG=C date +%H:%M") "\n"))
"Prepares Homeros menu text to show current time, date etc")
(defcustom homeros-menu-content '(
    ("Personal home directory" (lambda () (find-file "~")))
    ("Calendar" calendar)
    ("New message" message-mail)
    ("Gnus" gnus)
    ("W3M" w3m)
) "Homeros menu content list")
(defvar homeros-menu-actions nil "List of actions of the last menu appearance")
(defvar homeros-menu-start 0 "The first line number of menu items")
(defvar homeros-menu-map nil "Key map for homeros-menu")
(let ((map (make-sparse-keymap)))
  (suppress-keymap map t)
  (define-key map "\r" 'homeros-menu-run)
  (setq homeros-menu-map map))

(defun homeros-menu ()
  "Shows the Homeros menu buffer for quick user actions"
  (interactive)
  (set-buffer (get-buffer-create "homeros-menu"))
  (switch-to-buffer "homeros-menu")
  (setq buffer-read-only nil)
  (erase-buffer)
  (insert homeros-menu-header "\n\n")
  (when (functionp homeros-menu-top-text)
    (funcall homeros-menu-top-text))
  (setq homeros-menu-start (+ (count-lines 1 (point))
    (if (= (current-column) 0) 1 0)))
  (setq homeros-menu-actions '())
  (let ((q homeros-menu-content))
    (while q
      (insert (caar q) "\n")
      (add-to-list 'homeros-menu-actions (car (cdr (car q))) t (lambda (p1 p2) nil))
      (setq q (cdr q))))
  (setq major-mode 'homeros-menu-mode)
  (setq mode-name "homeros-menu-mode")
  (setq buffer-read-only t)
  (goto-line homeros-menu-start)
  (beginning-of-line)
  (use-local-map homeros-menu-map)
  (message homeros-menu-message))

(defun homeros-menu-run ()
  (interactive )
  (let ((ln) (q))
    (setq ln (+ (count-lines 1 (point))
      (if (= (current-column) 0) 1 0)))
    (setq ln (- ln homeros-menu-start))
    (when (>= ln 0)
      (setq q homeros-menu-actions)
      (while (not (equal ln 0))
	(setq q (cdr q))
	(setq ln (- ln 1)))
      (setq q (car q))
      (when q
	(kill-buffer (get-buffer "homeros-menu"))
	(funcall q)))))

(define-key function-key-map "\e[S" [(winkey)])
(global-set-key '[winkey] 'homeros-menu)

(provide 'homeros-menu)
